'	-------------------------------------------------------------
'	Name	: ILI9341_9163.inc
'	Author	: 
'	Date	: 28.2.2021
'	Version : 2.0
'	Notes	: Driver for ILI9341, IlI9163
'	-------------------------------------------------------------
'
'	$define TFT_Use_SPI
'	$define TFT_Use_Parallel
'	$define Touch_Use_SPI

'
'	-------------------------------------------------------------
' 
$ifndef _device 18F25K22
	Device 18F25K22
	$endif
     
$ifdef TFT_Use_SPI
'	Port constants
'	-----------------------------------------------------------------------------

    Symbol TFT_RST PORTC.0
	Symbol TFT_CS PORTC.1
	Symbol TFT_D_C PORTC.2
	Symbol CLK PORTC.3
	Symbol MOSI PORTC.5
	Symbol MISO PORTC.4

    TRISC = $10
	PORTC = 0

'	-----------------------------------------------------------------------------
'	SPI1 setup
'	----------------------------------------------------------------------------
	Symbol BF = SSP1STAT.0
	
	SSP1CON1 = %00110000				' SPI Master, FOSC/4 SCL 12MHz
'	SSP1CON1 = %00110001				' SPI Master, FOSC/16 SCL 3MHz
'	SSP1CON1 = %00110010				' SPI Master, FOSC/64 SCL 0.75MHz
'	SSP1CON1 = %00110011				' SPI Master, TMR2
'				 ||____________________  CKP  Clock Polarity Select
'				 |_____________________  SSPEN MSSP Enable

	SSP1STAT = %0000000
'				||______________________  CKE  Clock Edge Select
'				|_______________________  SMP  Sample At middle of Data Output time


GoTo Touch

'	SPI comands
'	--------------------------------------------------------------------------------

$define SPI_WR(s_byte_out, s_byte_in)		'
	SSP1BUF = s_byte_out					    '
	Repeat									'
	Until SSP1STAT.0 = 1					    '
	s_byte_in = SSP1BUF	

$define WriteCommand(command)				'
	TFT_D_C = 0								'
	SPI_WR(command,WREG)					'
	TFT_D_C = 1 
    
$define WriteData(data)					    '
	SPI_WR(data,WREG)


'-----------------------------------------------------------------------------------
' Macro Name    : ReadSPI
' Return Value  : Contents of SSPBUF register
' Parameters    : None
' Description   : Read a single byte from the SPI bus.
'
$define ReadData() _ReadSPI

_ReadSPI Macro- \Byte
    GoSub __ReadSPI
    Return_WREG
Endm

#ifMacro- _ReadSPI
__ReadSPI:
    WREG = SSP1BUF                   ' Empty the SSPBUF buffer and clear flags
    PIR1bits_SSPIF = 0              ' Clear interrupt flag
    SSP1BUF = $FF                    ' Initiate bus cycle
    Btfss PIR1bits_SSPIF            ' \
    Bra $ - 2                       ' / Wait until cycle complete
    WREG = SSP1BUF
    Return
#endIfMacro-

$ifdef TFT_Use_Parallel
	
	Symbol TFT_RST PORTA.0 
	Symbol TFT_CS PORTA.1
	Symbol TFT_RS PORTA.2	
	Symbol TFT_WR PORTA.3
	Symbol TFT_RD PORTA.5
	
	Output PORTA
    Output PORTB
    Output PORTC
    PORTA = $ff
    PORTB = $ff
    PORTC = $ff

GoTo Touch  

	$define WriteCommand wc
	Proc wc(command As Byte)
		TFT_RS = 0
        'PORTB = command						'
		PORTB.4 = GetBit command,0		'
		PORTB.5 = GetBit command,1  	'
		PORTB.6 = GetBit command,2		'
		PORTB.7 = GetBit command,3		'
		PORTC.0 = GetBit command,4		'
		PORTC.1 = GetBit command,5		'
		PORTC.2 = GetBit command,6		'
		PORTC.4 = GetBit command,7		'
		TFT_WR = 0						'
		TFT_WR = 1						'
		TFT_RS = 1
    EndProc
    	
	$define WriteData wd
	Proc wd(dato As Byte)
        ' PORTB = dato
		PORTB.4 = GetBit dato,0 	'
		PORTB.5 = GetBit dato,1		'
		PORTB.6 = GetBit dato,2		'
		PORTB.7 = GetBit dato,3		'
		PORTC.0 = GetBit dato,4		'
		PORTC.1 = GetBit dato,5		'
		PORTC.2 = GetBit dato,6		'
		PORTC.4 = GetBit dato,7		'
		TFT_WR = 0					'
		TFT_WR = 1
     EndProc
     		
	$define ReadData(data)					'
		TFT_RD = 0							'
		PORTB = data						'
		TFT_RD = 1
$endif

Touch:

$ifdef  Touch_Use_SPI
'	SPI setup
'	----------------------------------------------------------------------------
'	SCK = PORTB.1
'	SDI = PORTB.0
'	SDO = PORTC.7
	TRISB = %00000001
	Symbol ADS_CS_Pin PORTC.5			' SPI CS to ADS7846 CS (Active Low)

	SSPCON1  = %00110001	     			' SPI Master, FOSC/16 SCL 3MHz
'				  ||____________________  CKP  Clock Polarity Select
'				  |_____________________  SSPEN MSSP Enable
	SSPSTAT = %0100000
'				||______________________  CKE  Clock Edge Select
'				|_______________________  SMP  Sample At middle of Data Output time

'	SPI comand
'	--------------------------------------------------------------------------------
    High ADS_CS_Pin

GoTo over_lcd_subs 

    $define Touch_SPI_WR(s_byte_out, s_byte_in)		'
	SSPBUF = s_byte_out				    	'
	Repeat								     	'
	Until SSPSTAT.0 = 1					        '
	s_byte_in = SSPBUF

$endif

 GoTo over_lcd_subs
  	
'	--------------------------------------------
'	ILI9341 constants
'	--------------------------------------------

	Symbol Nop1			$00	' No operation
	Symbol SWRESET		$01	' Soft Reset

	Symbol RDDIDIF		$04 ' Read Display Identification Information
	Symbol RDDST		$09 ' Read Display Status
	Symbol RDDPM		$0A	' Read Display Power Mode
	Symbol RDDMADCTL	$0B	' Read Display MADCTL
	Symbol RDDCOLMOD	$0C ' Read Display Pixel Format (0Ch)
	Symbol RDDIM		$0D	' Read Display Image Mode
	Symbol RDDSM		$0E	' Read Display Signal Mode)
	Symbol RDDSDR		$0F ' Read Display Self-Diagnostic Result

	Symbol SLPIN		$10	' Sleep ON
	Symbol SLPOUT		$11	' Sleep OFF
	Symbol PTLON		$12	' Partial Mode ON
	Symbol NORON		$13	' Normal Display ON
	Symbol DINVOFF		$20	' Display Inversion OFF
	Symbol DINVON		$21	' Display Inversion ON
	Symbol GAMMASET		$26	' Gamma Set ($01[1],$02[2],$04[3],$08[4])
	Symbol DISPOFF		$28	' Display OFF
	Symbol DISPON		$29	' Display ON

	Symbol CASET		$2A ' Column Address Set
	Symbol RASET		$2B ' Row Address Set
	Symbol RAMWR		$2C	' Memory Write
	Symbol RGBSET		$2D	' Color Set : 4K/65K/262K
	Symbol RAMRD		$2E	' Memory Read
	Symbol PLTAR		$30	' Partial Area
	Symbol VSCRDEF		$33	' Vertical ScrollING Definition
	Symbol TEOFF		$34	' Tearing Effect Line OFF
	Symbol TEON			$35	' Tearing Effect Line ON
	Symbol MADCTL		$36	' Memory Access Control
	Symbol VSCRSADD		$37	' Vertical Scrolling Start address
	Symbol IDMOFF		$38	' Idle Mode OFF
	Symbol IDMON		$39	' Idle Mode ON
	
  	Symbol PIXSET		$3A	' Pixel Format Set
	Symbol WRDISBV		$51 ' Write Display Brightness
	Symbol RDDISBV		$52	' Read Display Brightness
	Symbol WRCTRLD		$53	' Write Control Display
	Symbol RDCTRLD		$54	' Read Control Display
	Symbol WRCABC		$55	' Write Content Adaptive Brightness Control
	Symbol RDCABC		$56	' Read Content Adaptive Brightness Control
	Symbol FRMCTR1		$B1	' Frame Rate Control (In normal mode/Full colors)
	Symbol FRMCTR2		$B2	' Frame Rate Control(In Idle mode/8-colors)
	Symbol FRMCTR3		$B3	' Frame Rate Control(In Partial mode/full colors)
	Symbol INVTR		$B4	' Display Inversion Control
	Symbol PRDTR		$B5	' RGB Interface Blanking Porch setting
	Symbol DISCTRL		$B6	' Display Fuction Control
	Symbol ETMOD		$B7	' Entry Mode Set
	Symbol GDRVDIR		$B8	' Gate Driver Direction Control

	Symbol PWCTR1		$C0	' Power_Control1
	Symbol PWCTR2		$C1	' Power_Control2
	Symbol VCOMCTRL1	$C5	' VCOM_Control 1
	Symbol VCOMCTRL2	$C6	' VCOM_Control 2
	Symbol VCOMOFFS		$C7	' VCOM Offset Control
	Symbol PGAMCTRL		$E0	' Positive Gamma Correction Setting
	Symbol NGAMCTRL		$E1	' Negative Gamma Correction Setting
	Symbol GAMRSEL		$F2	' GAM_R_SEL

	Symbol true=1
	Symbol false=0

'	-------------------------------------------------------------

$ifdef ILI9163
	Symbol OR_0		$40
	Symbol OR_90	$20
	Symbol OR_180	$80
	Symbol OR_270	$E0
	Symbol DISPLAY_WIDTH 128
	Symbol DISPLAY_HEIGHTH 160
$endif

$ifdef ILI9341
	Symbol OR_0		$48'50
 	Symbol OR_90	$28
	Symbol OR_180	$88
	Symbol OR_270	$E8
	Symbol DISPLAY_WIDTH 240
	Symbol DISPLAY_HEIGHTH 320
$endif

'	-------------------------------------------------------------

	Symbol iBit = STATUS.0	' Carry/Borrow bit

	Dim wFSR0 As FSR0L.Word
    Dim wFSR1 As FSR1L.Word
    Dim wFSR2 As FSR2L.Word 
   
    Dim des As Bit
	Dim wSCRHeigth As Word				' Screen size
	Dim wSCRWidth As Word
	Dim wXstart As Word					' X-Coordinate
	Dim wYstart As Word					' Y-Coordinate
	Dim wXend As Word					' X-Coordinate
	Dim wYend As Word					' Y-Coordinate
	Dim wWidth As Word
	Dim wHeight As Word
	Dim wInk As Word
	Dim wPaper As Word
	Dim wColor As Word
	Dim wNumBits As Dword				' Nubmer of bits. Word is enough for ILI9163
	Dim br As Byte
	Dim bg As Byte
	Dim bb As Byte

'	PrintString variables
'	-------------------------------------------------------------

	Dim wPrintStr As String * 50		' String used in PrintString
	Dim bChrPtr As Word					' Pointer to lookup table
	Dim wFontAddress As Word
	Dim wChrAddr As Word				' Characer address
	Dim wChrAdr As Word
	Dim wChrBytes As Word				' Number of bytes in character
	Dim bFirstChr As Byte
	Dim bLastChr As Byte
	Dim bCOLS As Byte					' Columns of charcter, used in PrintString
	Dim bCharWidth As Byte				' Same as bCols, used to exit bBitLoop
	Dim bROWS As Byte					' Rows of charcter.
    Dim bRowBytes As Byte
    Dim bColBytes As Byte
	Dim bCDATA_BYTE As Byte				' Data byte read from font
	Dim bStrLoop As Byte				' Loops through wPrintStr
	Dim bBitLoop As Byte				' Sends bits. Exits when charwidth is zero
	Dim BMPadress As Word
	Dim c As Byte 
    Dim h As Word
    Dim w As Word

	

'	Initialization
'	--------------------------------------------------------------------------------
'	The init routine is very simple, only 10 commands. This is in contrast with other
'	init routines that I have seen that contain "gazillions of commands" which
'	use among other things "sparkles and rainbows" and "magic unicorn dust".
'	This one only starts the display.

$define InitILI _InitILI()
	Proc _InitILI()  
		TFT_CS = 0							' GLCD ON
		TFT_RST = 0
		DelayMS 120
		TFT_RST = 1                            
		DelayMS 120
		WriteCommand(SWRESET)				' Software reset
		WriteCommand(SLPOUT)				' Exit sleep 
		WriteCommand(PIXSET)				' Set Color Format
		WriteData($05)						' 16bit
		WriteCommand(GAMMASET)				' Set gamma curve
		WriteData($01)						' $01[1],$02[2],$04[3],$08[4]
		WriteCommand(GAMRSEL)				' Enable Gamma adj
		WriteData($01)
		WriteCommand(NORON)					' Normal display
		WriteCommand(DISPON)				' Display ON
		
	EndProc


'	Commands to set screen orientation, size of screen,
'	color order to RGB and printing from left to right.
'	---------------------------------------------------

$define SetMADCTL(sor)						'
	TFT_CS = 0								'
	WriteCommand(MADCTL)					'
	WriteData(sor)							'
	TFT_CS = 1

 
$define SCRorientation0						'
	SetMADCTL(OR_0)                         '
    wSCRWidth =  DISPLAY_WIDTH				'
	wSCRHeigth =DISPLAY_HEIGHTH

$define SCRorientation90					'
	SetMADCTL(OR_90)                        '
    wSCRWidth =  DISPLAY_HEIGHTH			'
	wSCRHeigth = DISPLAY_WIDTH	

$define SCRorientation180					'
	SetMADCTL(OR_180)                       '
    wSCRWidth =  DISPLAY_WIDTH				'
	wSCRHeigth = DISPLAY_HEIGHTH

$define SCRorientation270					'
	SetMADCTL(OR_270)                       '
    wSCRWidth =	 DISPLAY_HEIGHTH     		'
	wSCRHeigth = DISPLAY_WIDTH	            '


'	RowColumnSet
'	---------------------------------------------------
'	Used in all commands that write or read screen to
'	define a rectangle to be written or read.
'	Size can be one pixel to full screen.

$define RowColumnSet _RowColumnSet()

Proc _RowColumnSet()
	WriteCommand(RASET)						'
	WriteData(wYstart.Byte1)				'
	WriteData(wYstart.Byte0)				'
	WriteData(wYend.Byte1)					'
	WriteData(wYend.Byte0)					'
	WriteCommand(CASET)						'
	WriteData(wXstart.Byte1)				'
	WriteData(wXstart.Byte0)				'
	WriteData(wXend.Byte1)					'
	WriteData(wXend.Byte0)
EndProc

'	---------------------------------------------------
'
'	ClearScreen
'	SetPixel(x,y)
'	XBlock(x,y,w,h)
'	HLine(x,y,w)
'	VLine(x,y,h)
'	XBox(x,y,w,h)
'
'	---------------------------------------------------
'	These routines all use RowColumnSet to set a
'	rectangle that is then filled. Size can be
'	one pixel = SetBit to full screen = ClearScreen.
'	Uses wColor.

$define ClearScreen							'
	XBlock(0,0,wSCRWidth,wSCRHeigth)

$define XBlock(x,y,w,h)						'
	wXstart = x : wYstart = y				'
	wXend = wXstart + w						'
	wYend = wYstart + h						'
	wNumBits = (w+1) * (h+1)				'
	subBLOCK

$define XBlock_CS(x,y,w,h)					'
	wXstart = x : wYstart = y				'
	wXend = wXstart + w						'
	wYend = wYstart + h						'
	wNumBits = (w+1) * (h+1)				'
	subBLOCK2
	
$define HLine(x,y,length)					'
	wXstart = x : wYstart = y				'
	wXend = wXstart + length				'
	wYend = wYstart							'
	wNumBits = length						'
	subBLOCK

$define VLine(x,y,length)					'
	wXstart = x : wYstart = y				'
	wXend = wXstart							'
	wYend = wYstart + length				'
	wNumBits = length						'
	subBLOCK

$define HLine2(x,y,length)					'
	wXstart = x : wYstart = y				'
	wXend = wXstart + length - 1			'
	wYend = wYstart							'
	wNumBits = length						'
	subBlock2

$define VLine2(x,y,length)					'
	wXstart = x : wYstart = y				'
	wXend = wXstart							'
	wYend = wYstart + length - 1			'
	wNumBits = length						'
	subBlock2

$define XBox2(x,y,x1,y1)					'
	TFT_CS = 0								'
	HLine2(x,y,x1-x)							'
	VLine2(x,y,y1-y)							'
	VLine2(x1,y,y1-y)						'
	HLine2(x,y1,x1-x)						'
	TFT_CS = 1
	
$define XBox(x,y,w,h)						'
	HLine(x,y,w)							'
	VLine(x,y,h)							'
	VLine(x+w,y,h)							'
	HLine(x,y+h,w)

$define pXBox _pXBox	
Proc _pXBox(pxstart As Byte,pystart As Byte,pwidth As Byte,pheight As Byte)
	TFT_CS = 0								'
	HLine2(pxstart,pystart,pwidth)			'
	VLine2(pxstart,pystart,pheight)			'
	VLine2(pxstart+pwidth,pystart,pheight)	'
	HLine2(pxstart,pystart+pheight,pwidth)
    TFT_CS = 1
 EndProc
    
'	SetPixel(x,y)
'	-------------------------------------
'	Needs TFT_CS = 0 prior to and TFT_CS = 1
'	after calling.

$define SetPixel _SetPixel
           
Proc _SetPixel(x,y)  				 		'
	wXstart = x : wYstart = y				'
	wXend = wXstart							'
	wYend = wYstart							'
	RowColumnSet							'
	WriteCommand(RAMWR)						'
	WriteData(wColor.Byte1)					'
	WriteData(wColor.Byte0)
 EndProc
 
'	subBLOCK
'	---------------------------------------------------
'	Defines and fills block.

Sub subBlock
	TFT_CS = 0
	RowColumnSet
	WriteCommand(RAMWR)
    For wNumBits = wNumBits To 1 Step - 1
	WriteData(wColor.Byte1)
	WriteData(wColor.Byte0)
	Next
	TFT_CS = 1
	EndSub

Sub subBlock2
	RowColumnSet
	WriteCommand(RAMWR)
	For wNumBits = wNumBits To 1 Step - 1
		WriteData(wColor.Byte1)
		WriteData(wColor.Byte0)
	Next
	EndSub	

'	SetupScrollArea(TFA,BFA)
'	----------------------------------------------------------------------------
'	Vertical Scrolling Definition (33h).	
'	wTopFixedArea = TFA						' Number of pixels from top	
'	wBottomFixedArea = BFA					' Number of pixels from bottom	
'	wVerticalScrollArea = DISPLAY_HEIGHTH - wBottomFixedArea - wTopFixedArea
	  
$define SetupScrollArea _SetupScrollArea
Proc _SetupScrollArea(TFA As wTopFixedArea ,BFA As wBottomFixedArea)
	Global Dim wTopFixedArea As Word		' Number of pixels from top		
	Global Dim wBottomFixedArea As Word		' Number of pixels from bottom	
	Global Dim wVerticalScrollArea As Word	' What is left	
	Global Dim wScrollStartAddress As Word	'	
	wVerticalScrollArea = DISPLAY_HEIGHTH - wBottomFixedArea - wTopFixedArea    '
	TFT_CS = 0								'
	WriteCommand(VSCRDEF)					'
	WriteData(wTopFixedArea.Byte1)			'
	WriteData(wTopFixedArea.Byte0)			'
	WriteData(wVerticalScrollArea.Byte1)	'
	WriteData(wVerticalScrollArea.Byte0)	'	
	WriteData(wBottomFixedArea.Byte1)		'
	WriteData(wBottomFixedArea.Byte0)		'	
	TFT_CS = 1
EndProc


'	ScrollAddress(SSA)
'	----------------------------------------------------------------------------
'	This command is used together with Vertical Scrolling Definition (33h).
'	These two commands describe the scrolling area and the scrolling mode.
'	The Vertical Scrolling Start Address command has one parameter which
'	describes the address of the line in the Frame Memory that will be written
'	as the first line after the last line of the Top Fixed Area
	
$define ScrollAddress _ScrollAddress
Proc _ScrollAddress(SSA As wScrollStartAddress)
	TFT_CS = 0								'
	WriteCommand(VSCRSADD)					'
	WriteData(wScrollStartAddress.Byte1)	'	
	WriteData(wScrollStartAddress.Byte0)	'	
	TFT_CS = 1
EndProc

$define GetPixel _GetPixel
Proc _GetPixel(pXstart As wXstart,pYstart As wYstart) , Word

    TFT_CS = 0
    
   	WriteCommand(PIXSET)				' Set Color Format
	WriteData($06)

    wYstart = pYstart				'
	wYend = pYstart					'
	wXstart = pXstart				'
	wXend = pXstart					'

    RowColumnSet
	WriteCommand(RAMRD)
	
    Input MOSI
    
    br = ReadData()
    
    br = ReadData()
    bg = ReadData()
    bb = ReadData()
    
    Result = (br & $3E) << 10
    Result = Result | ((bg & $3F) << 5)
    Result = Result | ((bb & $3E) >> 1)
    
    TFT_CS = 1
     
    
    Output MOSI
    
    TFT_CS = 0


	WriteCommand(PIXSET)				' Set Color Format
	WriteData($05)
   
    TFT_CS = 1

EndProc

'	Display_BMP(x,y,bmp)
'	----------------------------------------------------------------------------
'	Displays bmp with number of pixels, width and height in header
'

$define Display_BMP _Display_BMP
Proc _Display_BMP(pXstart As wXstart,pYstart As wYstart,BycRef BMPadress As Word)
	wNumBits = cPtr16(BMPadress++)				'
	wWidth = cPtr16(BMPadress++)				'
	wHeight = cPtr16(BMPadress++)
	wXend = wXstart + wWidth						'
	wYend = wYstart + wHeight	
	TFT_CS = 0
	RowColumnSet
	WriteCommand(RAMWR)
	For wNumBits = wNumBits To 1 Step - 1
		wColor = cPtr16(BMPadress++)
		WriteData(wColor.Byte1)
		WriteData(wColor.Byte0)
	Next
	TFT_CS = 1
EndProc



'	DisplayRAM_BMP(x,y,bmp)
'	----------------------------------------------------------------------------
'	Displays bmp with number of pixels, width and height in header
'

$define DisplayRAM_BMP _DisplayRAM_BMP
Proc _DisplayRAM_BMP(pXstart As wXstart, pYstart As wYstart, ByRef pBMPadress As BMPadress)
	wNumBits = Ptr16(BMPadress++)				'
	wWidth = Ptr16(BMPadress++)				'
	wHeight = Ptr16(BMPadress++)
	wXend = wXstart + wWidth
	wYend = wYstart + wHeight	
	TFT_CS = 0
	RowColumnSet
	WriteCommand(RAMWR)
	wNumBits = wNumBits * 2
	For wNumBits = wNumBits To 1 Step - 1
		WriteData(POSTINC1)
	Next
	TFT_CS = 1
EndProc



'	SetFont
'	----------------------------------------------------------------------------
'	Fonts are as defined in AN1182 by Microchip

$define SetFont _SetFont
Proc _SetFont(pFontAddress As wFontAddress)								'
	bFirstChr = CRead wFontAddress + 2
	bLastChr = CRead wFontAddress + 4
	bROWS = CRead wFontAddress + 6				' Height of the character
EndProc


$define At_XY(x,y)								'
	wXstart = x									'
	wYstart = y

$define PrintString(s)							'
	wPrintStr = s								'
	Print_string

Sub Print_String
	wFSR1 = AddressOf(wPrintStr)				' String to be printed
	TFT_CS = 0
	bStrLoop = Len (wPrintStr)-1				' Number of characters
	For bStrLoop = bStrLoop To 0 Step - 1
		bChrPtr = POSTINC1 - bFirstChr			' Get pointer to char
		If bChrPtr < bFirstChr Then Next		' If character is not in font
		If bChrPtr > bLastChr Then Next			' next char
		Print_Chr()	
	Next
	TFT_CS = 1
EndSub

$define Print_Char _Print_Car
    
Proc _Print_Car(c As Byte)
   	TFT_CS = 0
   	bChrPtr = c - 32
        Print_Chr()   
   	TFT_CS = 1
EndProc   

'	Print_Chr
'	---------------------------------------------------
'	Prints one character and increments wXstart
'	Used in PrintString
	
Proc Print_Chr()
	bChrPtr = bChrPtr * 4 + 8					' Each poiner is four bytes starting at address eight
	wChrAddr = wFontAddress + bChrPtr			' 
	bCOLS = cPtr8(wChrAddr)						' Width of the character
 	wChrAddr = CRead wChrAddr + 1				'
	wChrAddr = wChrAddr + wFontAddress			' Address of character
	bRowBytes = bCOLS / 8						' Find number of bytes in a row
	If bCOLS // 8 > 0 Then
		Inc bRowBytes
	EndIf
	wChrBytes = bRowBytes * bROWS				' Number of bytes in a caracter
	wXend = bCOLS - 1
	wXend = wXend + wXstart						' End col
	wYend = bROWS - 1
	wYend = wYend + wYstart						' End row
	RowColumnSet								' Define rectangle
	WriteCommand(RAMWR)
	bCharWidth = bCOLS							' Char width
	For wChrBytes = wChrBytes To 1 Step -1		' Print one character
		bCDATA_BYTE = cPtr8(wChrAddr)			' Read byte
		For bBitLoop = 1 To 8       			' Send 8 pixels
			Ror bCDATA_BYTE						' Rotate right
			If iBit = 1 Then					' iBit is the carry flag
				WriteData(wInk.Byte1)
				WriteData(wInk.Byte0)
			Else
				WriteData(wPaper.Byte1)
				WriteData(wPaper.Byte0)
			EndIf
			Dec bCharWidth
			If bCharWidth = 0 Then				' If required number of pixels have been sent
				bCharWidth = bCOLS				' restore bCharWidth
				Break							' exit bBITLoop
			EndIf
		Next
		Inc wChrAddr							' Next byte
	Next
	wXstart = wXstart + bCOLS					' Start of next char
EndProc

over_lcd_subs:

